/***************************************************************************
 *   Copyright (C) 2005-2013 LpzRobots development team                    *
 *   Authors:                                                              *
 *    Simón Smith <artificialsimon at ed dot ac dot uk>                    *
 *    Georg Martius  <georg dot martius at web dot de>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

// Simulation
#include <ode_robots/simulation.h>
// Noise generator
#include <selforg/noisegenerator.h>
// Agent: bind robot, controller and wiring together
#include "odeagent.h"
// The robot
#include "differential.h"
// Speed sensor
#include <ode_robots/speedsensor.h>

// Robot's controller
#include "swarmcontroller.h"
#include "basiccontroller.h"

//Add passive obstacles (balls)
#include <ode_robots/passivesphere.h> 

// Robot's wiring
#include <selforg/one2onewiring.h>
// Environmnet and obstacles

//#include <ode_robots/octaplayground.h>
#include <ode_robots/playground.h>    //Für einen Rechteckigen Playground

#include <ode_robots/passivebox.h>

using namespace lpzrobots;
using namespace std;

class ThisSim : public Simulation
{
  public:
    ThisSim() {
      // set Title of simulation
      setTitle("Project Sim KTZ");
    }
    ~ThisSim() { }
    BasicController* leader;
    Differential* leadingRobot;
    Differential* robot;
    OdeAgent* agent;
    One2OneWiring* wiring;
    const static int NumberofRobots = 26;
    SwarmController* follower[NumberofRobots];
    Differential *Swarm[NumberofRobots];

  virtual void start(const OdeHandle& odeHandle, const OsgHandle& osgHandle, GlobalData& global)
  {
    setCameraHomePos(Pos(0,0, 200),  Pos(0, 10, 0));
    global.odeConfig.setParam("controlinterval", 1);
    global.odeConfig.setParam("gravity", -9.8);
    DifferentialConf conf = Differential::getDefaultConf();
    conf.wheelMass = .5;
    std::vector<Differential*> Leader;
    double robotposX;
    double robotposY;
    double squaresize = 40;
    int square = (int) sqrt(NumberofRobots);
      
    for (int i = 0; i < NumberofRobots; i++)
    {
      std::string RobotName; // = "Differential robot xx";
      char buffer [23];

      robotposY = - 80 - squaresize/2 - (i-1)/square * squaresize/square;
      robotposX = -squaresize/2 + (i-1)%square * squaresize/square;
       
      sprintf (buffer, "Differential robot %d", i);
      if (i == 0)
      { 
        leadingRobot = new Differential(odeHandle, osgHandle, conf, buffer);
        leadingRobot->addSensor(std::make_shared<SpeedSensor>(1), Attachment(-1));
        leadingRobot->place(Pos(-4., -92., .5));
        leader = new BasicController("LeadingController");
        wiring = new One2OneWiring(new ColorUniformNoise(.1));
        agent = new OdeAgent(global);
        leader->meineBoolvar=true;  

        agent->init(leader, leadingRobot, wiring);
        global.agents.push_back(agent);
        global.configs.push_back(agent);
        Swarm[i]=leadingRobot;  
      }
      else
      {
        robot = new Differential(odeHandle, osgHandle, conf, buffer, NumberofRobots);
        robot->addSensor(std::make_shared<SpeedSensor>(1), Attachment(-1));
        robot->place(Pos( robotposX, robotposY, .5));
        follower[i] = new SwarmController("Swarm Controller",NumberofRobots);
        wiring = new One2OneWiring(new ColorUniformNoise(.1));
        agent = new OdeAgent(global);
        agent->init(follower[i], robot, wiring);
        global.agents.push_back(agent);
        global.configs.push_back(agent);
        Swarm[i]=robot;
        }
      }
      
      for(int p=0; p < NumberofRobots; p++)
      {
        Swarm[p]->Differential::RefSetter(Swarm);
      }
      
      
      const int PlaygroundSize=500;
      auto playground = new Playground(odeHandle, osgHandle, osg::Vec3(PlaygroundSize, .2, 1.2), 1); 
    
      playground->setGroundColor(Color(.784, .784, .0));
      playground->setColor(Color(1., .784, .082, .3));
      playground->setPosition(osg::Vec3(.0, .0, .1));
      global.obstacles.push_back(playground);
      
      double OstacleSideLength = 4.;
      double ObstacleDensity = 0.02;
      const int NumberofObstacles= (int) ((PlaygroundSize*(PlaygroundSize))*ObstacleDensity)/(OstacleSideLength*OstacleSideLength);
      for(int i=0; i<NumberofObstacles; i++)
      {
        auto box = new PassiveBox(odeHandle, osgHandle.changeColor(Color(0.0,0.0,1.0)), osg::Vec3(OstacleSideLength, OstacleSideLength, OstacleSideLength), 800.);
        box->setPosition(osg::Vec3(-250+rand()%500,-250+rand()%500,0));
        global.obstacles.push_back(box);
      }
  }

    virtual void addCallback(GlobalData& globalData, bool draw, bool pause, bool control) 
    {
    if (!pause)
    {    
        if  ( 0.99 <= (leadingRobot->OdeRobot::getOrientation().val(0,0)) && (leadingRobot->OdeRobot::getOrientation().val(0,0)) <= 1 &&
          0.99 <= (leadingRobot->OdeRobot::getOrientation().val(1,1)) && (leadingRobot->OdeRobot::getOrientation().val(1,1)) <= 1)
    {
      leader->meineBoolvar=true;
      leader->meineBoolvar2=false;    
    }
      else if (0.99 > (leadingRobot->OdeRobot::getOrientation().val(0,0)) &&
             0.99 > (leadingRobot->OdeRobot::getOrientation().val(1,1)) && 
                0 < (leadingRobot->OdeRobot::getOrientation().val(1,0)) && 
                0 > (leadingRobot->OdeRobot::getOrientation().val(0,1)))
    {
      leader->meineBoolvar2=true;
      leader->meineBoolvar=false;
    }
    else 
      {
      leader->meineBoolvar=false;
      leader->meineBoolvar2=false;
    }
    double velocities[NumberofRobots][3];
    
      for (int i = 0; i < NumberofRobots; i++)
      {      
        velocities[i][0] = Swarm[i]->getMainPrimitive()->getVel().x();
        velocities[i][1] = Swarm[i]->getMainPrimitive()->getVel().y();
        velocities[i][2] = Swarm[i]->getMainPrimitive()->getVel().z();
      }
    
      for (int i = 1; i < NumberofRobots; i++)
      {       
        follower[i]->VelSetter(velocities);
        }
      }
    }

    virtual bool command (const OdeHandle&, const OsgHandle&, GlobalData& globalData, int key, bool down) 
    {
      return false;
    }

    virtual void bindingDescription(osg::ApplicationUsage & au) const {}
};

int main (int argc, char **argv)
{
  // New simulation
  ThisSim sim;
  // Simulation begins
  return sim.run(argc, argv) ? 0 : 1;
}
