/***************************************************************************
 *   Copyright (C) 2005-2013 LpzRobots development team                    *
 *   Authors:                                                              *
 *    Simón Smith <artificialsimon at ed dot ac dot uk>                    *
 *    Georg Martius  <georg dot martius at web dot de>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "swarmcontroller.h"

#include <assert.h>


using namespace std;
using namespace matrix;


static int NoRobots;
struct spatial{
  double distance;
  double angle;
  spatial(double distance, double angle)
   {
     this->distance = distance;
     this->angle = angle;
   }
};

SwarmController::SwarmController(const std::string& name,const int NumberofRobots): AbstractController(name, "1.0") 
{
  initialised=false;
  NoRobots = NumberofRobots;
  addParameterDef("threshold", &threshold, 0.85, 0, 1, "threshold for IR-sensor");
}

void SwarmController::SensorToPos(const sensor* sensors, spatial &PosVector, const int iterator)
{  
  char bufferX [18];
  char bufferY [18];
  char bufferZ [18];
  sprintf (bufferX, "relpossensor %iX", iterator);
  sprintf (bufferY, "relpossensor %iY", iterator);
  sprintf (bufferZ, "relpossensor %iZ", iterator);
  
  double X = sensors[SIdx(bufferX)];
  double Y = sensors[SIdx(bufferY)];
  double Z = sensors[SIdx(bufferZ)];
  PosVector.distance = sqrt(X*X+Y*Y+Z*Z);
  if(PosVector.distance < 0.1)
  {
    PosVector.angle = iterator;
  }
}


void SwarmController::getClosest(const sensor* sensors, double &angle)
{
  spatial PosVector(0,0);
  double alpha = 1; // "Temperature"
  double positionoffset=10.; // "Energy"
  
  for(int i=0;i<NoRobots;i++)
  {
    SensorToPos(sensors, PosVector, i);
  }
  
  int own = (int) PosVector.angle;
  double Vel[3] = {0};

  for (int i = 0; i < NoRobots; i++)
  {
    if (i == own) PosVector.distance=1;
    SensorToPos(sensors, PosVector, i); 
	  if (i == 0)  
	  { 
      Vel[0] += Velocities[i][0] * NoRobots*(1./(exp((PosVector.distance-positionoffset)/alpha)+1));
      Vel[1] += Velocities[i][1] * NoRobots*(1./(exp((PosVector.distance-positionoffset)/alpha)+1));
      Vel[2] += Velocities[i][2] * NoRobots*(1./(exp((PosVector.distance-positionoffset)/alpha)+1));
	  } 
	  else 
	  {
	    Vel[0] += Velocities[i][0] *(1./(exp((PosVector.distance-positionoffset)/alpha)+1));
      Vel[1] += Velocities[i][1] *(1./(exp((PosVector.distance-positionoffset)/alpha)+1));
      Vel[2] += Velocities[i][2] *(1./(exp((PosVector.distance-positionoffset)/alpha)+1));
	  }
	  angle = ( - atan2(Vel[0],Vel[1]) + atan2(Velocities[own][0],Velocities[own][1]))*(180./M_PI);
  }
}

void SwarmController::stepNoLearning(const sensor* sensors, int number_sensors, motor* motors, int number_motors) 
{
  double angle;          
  getClosest(sensors, angle);
  
  double veloffset = 2.;
  double temperature = .0;
  
  if (sensors[SIdx("IR front left")] > 2*threshold || sensors[SIdx("IR front right")] > 2*threshold) 
  { // move backward
    motors[MIdx("left motor")] = -1.;
    motors[MIdx("right motor")] = -1.;
  }
  else if (sensors[SIdx("IR left")] > threshold ||
            sensors[SIdx("IR left front")] > threshold ||
            sensors[SIdx("IR left left front")] > threshold ||
            sensors[SIdx("IR front front left")] > threshold ||
            sensors[SIdx("IR front left")] > threshold) 
  { // turn right
    motors[MIdx("left motor")] = -1.;
    motors[MIdx("right motor")] = 1.;
  }
  else if (sensors[SIdx("IR right")] > threshold ||
           sensors[SIdx("IR right front")] > threshold ||
	         sensors[SIdx("IR right right front")] > threshold ||
	         sensors[SIdx("IR front front right")] > threshold ||
           sensors[SIdx("IR front right")] > threshold) 
  { // turn left
    motors[MIdx("left motor")] = 1.;
    motors[MIdx("right motor")] = -1.;
  }
  else 
  { 
    motors[MIdx("left motor")]  = 1/(exp((-angle-veloffset)/temperature)+1);
    motors[MIdx("right motor")] = 1/(exp((+angle-veloffset)/temperature)+1);
  }
}

void SwarmController::step(const sensor* sensors, int sensornumber, motor* motors, int motornumber) 
{
  stepNoLearning(sensors,sensornumber, motors, motornumber);
}


void SwarmController::init(int sensornumber, int motornumber, RandGen* randGen) {
  assert(motornumber >=2 && sensornumber >=8);
  // Set the number of sensors and motors
  nSensors = sensornumber;
  nMotors  = motornumber;
  initialised=true;
}

void SwarmController::VelSetter(const double Velocities[][3])
{
  for (int i = 0; i < NoRobots; i++)
  {
    this->Velocities[i][0] = Velocities[i][0];
    this->Velocities[i][1] = Velocities[i][1];
    this->Velocities[i][2] = Velocities[i][2];
  }
}

int SwarmController::getSensorNumber() const {
  return nSensors;
}

int SwarmController::getMotorNumber() const {
  return nMotors;
}

bool SwarmController::store(FILE* f) const {
  //  S.store(f); // if S is a matrix::Matrix
  Configurable::print(f,0);
  return true;
}

bool SwarmController::restore(FILE* f) {
  //  S.restore(f); // if S is a matrix::Matrix
  Configurable::parse(f);
  return true;
}
